@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2018 Michal Trojnara <Michal.Trojnara@stunnel.org>
+ *   Copyright (C) 1998-2019 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -564,6 +564,9 @@ NOEXPORT LRESULT CALLBACK window_proc(HWND main_window_handle,
         case IDM_REOPEN_LOG:
             signal_post(SIGNAL_REOPEN_LOG);
             break;
+        case IDM_CONNECTIONS:
+            signal_post(SIGNAL_CONNECTIONS);
+            break;
         case IDM_MANPAGE:
 #ifndef _WIN32_WCE
             if(!cmdline.service) /* security */
